# Activate advanced markdown delimiters
from .markdown_ext import markdown

# Core parsers
from .custom_markdown import CustomMarkdown
from .html_ext import CustomHtmlParser

# Unified API
from .api import parse, unparse

__all__ = [
    "markdown",
    "CustomMarkdown",
    "CustomHtmlParser",
    "parse",
    "unparse",
]

"""
📦 Delimiters advanced add-ons for Telethon
👤 Created by Ankit Chaubey | https://github.com/ankit-chaubey/delimiters
© 2026 Ankit Chaubey. All rights reserved.
📬 Contact: m.ankitchaubey@gmail.com | Telegram: @ankify @AnkitChaubey @ChaubeyBot
⚠️ Warning: Do not edit this file unless you are absolutely sure what you are doing.
💥 Changing this file may break the package.
🚀 Initially built for personal use, now released publicly.
"""
